/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.db;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Session;
import org.hibernate.tool.hbm2ddl.SchemaExport;
import org.jbpm.AbstractJbpmTestCase;
import org.jbpm.JbpmConfiguration;
import org.jbpm.JbpmContext;
import org.jbpm.graph.def.ProcessDefinition;
import org.jbpm.graph.exe.ProcessInstance;
import org.jbpm.logging.log.ProcessLog;
import org.jbpm.msg.db.TextMessage;
import org.jbpm.taskmgmt.exe.TaskInstance;

public class AbstractDbTestCase extends AbstractJbpmTestCase {
  
  protected static JbpmConfiguration jbpmConfiguration = 
           JbpmConfiguration.parseResource("org/jbpm/jbpm.test.cfg.xml");

  protected JbpmContext jbpmContext = null;
  protected SchemaExport schemaExport = null;
  
  protected Session session = null;
  protected GraphSession graphSession = null;
  protected TaskMgmtSession taskMgmtSession = null;
  protected ContextSession contextSession = null;
  protected SchedulerSession schedulerSession = null;
  protected LoggingSession loggingSession = null;
  protected MessagingSession messagingSession = null;
  
  public void setUp() throws Exception {
    super.setUp();
    createSchema();
    createJbpmContext();
    initializeMembers();
    
    log.debug("");
    log.debug("### starting "+getName()+" ####################################################");
  }

  public void tearDown() throws Exception {
    log.debug("### "+getName()+" done ####################################################");
    log.debug("");

    resetMembers();
    closeJbpmContext();
    dropSchema();
    super.tearDown();
  }

  public void beginSessionTransaction() {
    createJbpmContext();
    initializeMembers();
  }
  
  public void commitAndCloseSession() {
    closeJbpmContext();
    resetMembers();
  }
  
  protected void newTransaction() {
    try {
      commitAndCloseSession();
      beginSessionTransaction();
    } catch (Throwable t) {
      throw new RuntimeException("couldn't commit and start new transaction", t);
    }
  }

  public ProcessInstance saveAndReload(ProcessInstance pi) {
    jbpmContext.save(pi);
    newTransaction();
    return graphSession.loadProcessInstance(pi.getId());
  }
  public TaskInstance saveAndReload(TaskInstance taskInstance) {
    jbpmContext.save(taskInstance);
    newTransaction();
    return (TaskInstance) session.load(TaskInstance.class, new Long(taskInstance.getId()));
  }
  

  public ProcessDefinition saveAndReload(ProcessDefinition pd) {
    graphSession.saveProcessDefinition(pd);
    newTransaction();
    return graphSession.loadProcessDefinition(pd.getId());
  }
  public ProcessLog saveAndReload(ProcessLog processLog) {
    loggingSession.saveProcessLog(processLog);
    newTransaction();
    return loggingSession.loadProcessLog(processLog.getId());
  }
  public TextMessage saveAndReload(TextMessage message) {
    session.saveOrUpdate(message);
    newTransaction();
    return (TextMessage) session.load(TextMessage.class, new Long(message.getId()));
  }
  
  protected void createSchema() {
    getJbpmConfiguration().createSchema();
  }

  protected JbpmConfiguration getJbpmConfiguration() {
    return jbpmConfiguration;
  }
  
  protected void dropSchema() {
    getJbpmConfiguration().dropSchema();
  }

  protected void createJbpmContext() {
    jbpmContext = getJbpmConfiguration().createJbpmContext();
  }

  protected void closeJbpmContext() {
    jbpmContext.close();
  }
  
  protected void initializeMembers() {
    session = jbpmContext.getSession();
    graphSession = jbpmContext.getGraphSession();
    taskMgmtSession = jbpmContext.getTaskMgmtSession();
    loggingSession = jbpmContext.getLoggingSession();
    schedulerSession = jbpmContext.getSchedulerSession();
    contextSession = jbpmContext.getContextSession();
    messagingSession = jbpmContext.getMessagingSession();
  }
  
  protected void resetMembers() {
    session = null;
    graphSession = null;
    taskMgmtSession = null;
    loggingSession = null;
    schedulerSession = null;
    contextSession = null;
    messagingSession = null;
  }
  
  private static Log log = LogFactory.getLog(AbstractDbTestCase.class);
}
